--- old/usr/src/lib/libc/port/gen/getlogin.c	Mon Aug 12 18:24:54 2013
+++ new/usr/src/lib/libc/port/gen/getlogin.c	Mon Aug 12 18:24:54 2013
@@ -20,6 +20,8 @@
  */
 
 /*
+ * Copyright (c) 2013 Gary Mills
+ *
  * Copyright 2008 Sun Microsystems, Inc.  All rights reserved.
  * Use is subject to license terms.
  */
@@ -27,11 +29,9 @@
 /*	Copyright (c) 1988 AT&T	*/
 /*	  All Rights Reserved  	*/
 
-#pragma ident	"%Z%%M%	%I%	%E% SMI"
+#pragma weak _getlogin = getloginx
+#pragma weak _getlogin_r = getloginx_r
 
-#pragma weak _getlogin = getlogin
-#pragma weak _getlogin_r = getlogin_r
-
 #include "lint.h"
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -47,28 +47,42 @@
 #include <mtlib.h>
 #include "tsd.h"
 
+/* Revert the renames done in unistd.h */
+#ifdef	__PRAGMA_REDEFINE_EXTNAME
+#pragma	redefine_extname	getlogint	getlogin
+#pragma	redefine_extname	getlogint_r	getlogin_r
+#pragma	redefine_extname	__posix_getlogint_r	__posix_getlogin_r
+#else	/* __PRAGMA_REDEFINE_EXTNAME */
+#ifdef	getlogin
+#undef	getlogin
+#endif	/* getlogin */
+#ifdef	getlogin_r
+#undef	getlogin_r
+#endif	/* getlogin_r */
+#ifdef	__posix_getlogin_r
+#undef	__posix_getlogin_r
+#endif	/* __posix_getlogin_r */
+#define	getlogint	getlogin
+#define	getlogint_r	getlogin_r
+#define	__posix_getlogint_r	__posix_getlogin_r
+#endif	/* __PRAGMA_REDEFINE_EXTNAME */
+
 /*
- * XXX - _POSIX_LOGIN_NAME_MAX limits the length of a login name.  The utmpx
- * interface provides for a 32 character login name, but for the sake of
- * compatibility, we are still using the old utmp-imposed limit.
+ * Use the full length of a login name.
+ * The utmpx interface provides for a 32 character login name.
  */
+#define	NMAX	(sizeof (((struct utmpx *)0)->ut_user))
 
 /*
- * POSIX.1c Draft-6 version of the function getlogin_r.
- * It was implemented by Solaris 2.3.
+ * Common function
  */
-char *
-getlogin_r(char *answer, int namelen)
+static char *
+getl_r_common(char *answer, size_t namelen, size_t maxlen)
 {
 	int		uf;
 	off64_t		me;
 	struct futmpx	ubuf;
 
-	if (namelen < _POSIX_LOGIN_NAME_MAX) {
-		errno = ERANGE;
-		return (NULL);
-	}
-
 	if ((me = (off64_t)ttyslot()) < 0)
 		return (NULL);
 	if ((uf = open64(UTMPX_FILE, 0)) < 0)
@@ -81,24 +95,39 @@
 	(void) close(uf);
 	if (ubuf.ut_user[0] == '\0')
 		return (NULL);
-	(void) strncpy(&answer[0], &ubuf.ut_user[0],
-	    _POSIX_LOGIN_NAME_MAX - 1);
-	answer[_POSIX_LOGIN_NAME_MAX - 1] = '\0';
+
+	/* Insufficient buffer size */
+	if (namelen < strnlen(&ubuf.ut_user[0], maxlen)) {
+		errno = ERANGE;
+		return (NULL);
+	}
+	(void) strncpy(&answer[0], &ubuf.ut_user[0], maxlen);
+	answer[maxlen] = '\0';
 	return (&answer[0]);
 }
 
 /*
+ * POSIX.1c Draft-6 version of the function getlogin_r.
+ * It was implemented by Solaris 2.3.
+ */
+char *
+getlogint_r(char *answer, int namelen)
+{
+	return (getl_r_common(answer, (size_t)namelen, LOGNAME_MAX_TRAD));
+}
+
+/*
  * POSIX.1c standard version of the function getlogin_r.
  * User gets it via static getlogin_r from the header file.
  */
 int
-__posix_getlogin_r(char *name, int namelen)
+__posix_getlogint_r(char *name, int namelen)
 {
 	int nerrno = 0;
 	int oerrno = errno;
 
 	errno = 0;
-	if (getlogin_r(name, namelen) == NULL) {
+	if (getl_r_common(name, (size_t)namelen, LOGNAME_MAX_TRAD) == NULL) {
 		if (errno == 0)
 			nerrno = EINVAL;
 		else
@@ -109,11 +138,57 @@
 }
 
 char *
-getlogin(void)
+getlogint(void)
 {
-	char *answer = tsdalloc(_T_LOGIN, _POSIX_LOGIN_NAME_MAX, NULL);
+	char *answer = tsdalloc(_T_LOGIN, LOGIN_NAME_MAX_TRAD, NULL);
 
 	if (answer == NULL)
 		return (NULL);
-	return (getlogin_r(answer, _POSIX_LOGIN_NAME_MAX));
+	return (getl_r_common(answer, LOGIN_NAME_MAX_TRAD, LOGNAME_MAX_TRAD));
+}
+
+/*
+ * POSIX.1c Draft-6 version of the function getlogin_r.
+ * It was implemented by Solaris 2.3.
+ * For extended login names, selected by redefine_extname in unistd.h.
+ */
+char *
+getloginx_r(char *answer, int namelen)
+{
+	return (getl_r_common(answer, (size_t)namelen, NMAX));
+}
+
+/*
+ * POSIX.1c standard version of the function getlogin_r.
+ * User gets it via static getlogin_r from the header file.
+ * For extended login names, selected by redefine_extname in unistd.h.
+ */
+int
+__posix_getloginx_r(char *name, int namelen)
+{
+	int nerrno = 0;
+	int oerrno = errno;
+
+	errno = 0;
+	if (getl_r_common(name, (size_t)namelen, NMAX) == NULL) {
+		if (errno == 0)
+			nerrno = EINVAL;
+		else
+			nerrno = errno;
+	}
+	errno = oerrno;
+	return (nerrno);
+}
+
+/*
+ * For extended login names, selected by redefine_extname in unistd.h.
+ */
+char *
+getloginx(void)
+{
+	char *answer = tsdalloc(_T_LOGIN, LOGIN_NAME_MAX, NULL);
+
+	if (answer == NULL)
+		return (NULL);
+	return (getl_r_common(answer, LOGIN_NAME_MAX, NMAX));
 }
